! ------------------------------------------------------------------------
!>  \brief Module defining System Constants
! ------------------------------------------------------------------------
MODULE system_constants

IMPLICIT NONE

INTEGER, PARAMETER :: MAXLEN = 200            !> maximum string length

DOUBLE PRECISION, PARAMETER :: ONE_THIRD = 0.33333333333333333d0    !> avoid computing 1/3

INTEGER, PARAMETER :: NDIM          = 2       !> number of coordinate dims
INTEGER, PARAMETER :: NTNS          = 3       !> number of tensor components
INTEGER, PARAMETER :: NNODEL        = 3       !> number of nodes per body element
INTEGER, PARAMETER :: NNODELB       = 2       !> number of nodes per traction element
INTEGER, PARAMETER :: MAX_NODES     = 2000    !> maximum number of nodes
INTEGER, PARAMETER :: MAX_DOFS      = 3990    !> maximum number of degrees of freedom
INTEGER, PARAMETER :: MAX_ELEMENTS  = 5000    !> maximum number of elements
INTEGER, PARAMETER :: MAX_BOUNDELS  = 2000    !> maximum number of traction elements
INTEGER, PARAMETER :: MAX_MATERIALS = 30      !> maximum number of materials
INTEGER, PARAMETER :: MAX_TIMESTEPS = 10000   !> maximum number of time steps

DOUBLE PRECISION, PARAMETER :: E_MIN      =  0.d0       !> minimum value of elastic modulus
DOUBLE PRECISION, PARAMETER :: E_MAX      =  1.d11      !> maximum value of elastic modulus
DOUBLE PRECISION, PARAMETER :: NU_MIN     =  0.d0       !> minimum value of Poisson's ratio
DOUBLE PRECISION, PARAMETER :: NU_MAX     =  0.499d0    !> maximum value of Poisson's ratio
DOUBLE PRECISION, PARAMETER :: RHO_MIN    =  0.d0       !> minimum value of density
DOUBLE PRECISION, PARAMETER :: RHO_MAX    =  1.d11      !> maximum value of density
DOUBLE PRECISION, PARAMETER :: COORD_MIN  = -1.d11      !> minimum value of coordinates
DOUBLE PRECISION, PARAMETER :: COORD_MAX  =  1.d11      !> maximum value of coordinates
DOUBLE PRECISION, PARAMETER :: DISP_MIN   = -1.d11      !> minimum value of displacement
DOUBLE PRECISION, PARAMETER :: DISP_MAX   =  1.d11      !> maximum value of displacement
DOUBLE PRECISION, PARAMETER :: VEL_MIN    = -1.d11      !> minimum value of velocity
DOUBLE PRECISION, PARAMETER :: VEL_MAX    =  1.d11      !> maximum value of velocity
DOUBLE PRECISION, PARAMETER :: ACC_MIN    = -1.d11      !> minimum value of acceleration
DOUBLE PRECISION, PARAMETER :: ACC_MAX    =  1.d11      !> maximum value of acceleration
DOUBLE PRECISION, PARAMETER :: SIG_MIN    = -1.d11      !> minimum value of stress
DOUBLE PRECISION, PARAMETER :: SIG_MAX    =  1.d11      !> maximum value of stress
DOUBLE PRECISION, PARAMETER :: STR_SMALL  =  1.d-2      !> threshold for small strains
DOUBLE PRECISION, PARAMETER :: STR_MIN    = -STR_SMALL  !> minimum value of strain
DOUBLE PRECISION, PARAMETER :: STR_MAX    =  STR_SMALL  !> maximum value of strain
DOUBLE PRECISION, PARAMETER :: DTIME_MIN  =  1.d-11     !> minimum value of time step
DOUBLE PRECISION, PARAMETER :: DTIME_MAX  =  1.d4       !> maximum value of time step

END MODULE system_constants
